#!/usr/bin/python
#
# GUI for RHN Registration
# Copyright (c) 2000-2002 Red Hat, Inc.
#
# Author: Preston Brown <pbrown@redhat.com>


from socket import gethostname
from os import geteuid, chdir
import sys
import string

saveargs = []
for arg in sys.argv[1:]:
    saveargs.append(arg)
    sys.argv.remove(arg)

import gtk, GTK
from gtk import TRUE
from gtk import FALSE
import GDK
import gnome
import gnome.ui
import libglade
import signal

sys.path.append("/usr/share/rhn/register")

import progress
import checklist
import rhnreg, hardware

from translate import _, N_, cat

# wrap a long line...
def wrap_line(line, max_line_size = 100):
    if len(line) < max_line_size:
        return line
    ret = []
    l = ""
    for w in string.split(line):
        if not len(l):
            l = w
            continue
        if len(l) > max_line_size:
            ret.append(l)
            l = w
        else:
            l = "%s %s" % (l, w)
    if len(l):
        ret.append(l)
    return string.join(ret, '\n')
# wrap an entire piece of text
def wrap_text(txt):
    return string.join(map(wrap_line, string.split(txt, '\n')), '\n')

def errorWindow(msg):
    dlg = gnome.ui.GnomeErrorDialog(wrap_text(msg))
    dlg.run_and_close()

class Gui:

    def __init__(self):
        self.test = FALSE
        self.running = FALSE
        self.initPrivacy = FALSE
        self.initPackages = FALSE
        self.initProfile = FALSE
        self.lastRow = -1
        self.alreadyRegistered = FALSE
        self.xml = libglade.GladeXML("/usr/share/rhn/register/gui.glade",
                                     "mainWin", domain="rhn_register")
        self.oemInfo = {}
        self.productInfo = {}
        
        # set up signal/slot connections
        self.xml.signal_autoconnect (
            { "onDruidCancel" : self.onDruidCancel,
              "onPrivacyPagePrepare": self.onPrivacyPagePrepare,
              "onInfoPagePrepare" : self.onInfoPagePrepare,
              "onInfoPageNext" : self.onInfoPageNext,
              "onProductPagePrepare" : self.onProductPagePrepare,
              "onProductPageNext" : self.onProductPageNext,
              "onProfilePagePrepare" : self.onProfilePagePrepare,
              "onProfilePageNext" : self.onProfilePageNext,
              "onProfilePageBack" : self.onProfilePageBack,
              "onEnableHardwareButtonToggled" : self.onEnableHardwareButtonToggled,
              "onPackagePagePrepare" : self.onPackagePagePrepare,
              "onEnableRpmButtonToggled" : self.onEnableRpmButtonToggled,
              "onSendPageBack" : self.onSendPageBack,
              "onSendPageNext" : self.onSendPageNext,
              "onFinishPagePrepare" : self.onFinishPagePrepare,
              "onFinishPageFinish" : self.onFinishPageFinish })

        sp = self.xml.get_widget("startPage")
        self.mainWin = self.xml.get_widget("mainWin")                
        self.mainWin.connect("delete-event", gtk.mainquit)
        self.mainWin.connect("hide", gtk.mainquit)

        window = self.xml.get_widget("packageWindow")
        self.packageArea = checklist.CheckList(3)
        self.packageArea.set_column_title(1, _("Package Name"))
        self.packageArea.set_column_min_width(1, 200)
        self.packageArea.set_column_title(2, _("Version"))
        self.packageArea.set_column_title(3, _("Release"))
        self.packageArea.column_titles_show()
        #self.packageArea.connect("select_row", self.showPackageInfo)

        window.add(self.packageArea)
        window.show_all()

        
        self.setBusyCursor()
        try:
            sp.set_text(rhnreg.welcomeText())
        except rhnreg.CommunicationError, e:
            self.fatalError(_("Fatal error retrieving welcome message:\n") + e.errmsg)
        self.setArrowCursor()
        self.running = TRUE


    def onDruidCancel(self, dummy):
        gtk.mainquit()
        sys.exit(2)


    def fatalError(self, error):
        self.setArrowCursor()
        dlg = gnome.ui.GnomeErrorDialog(wrap_text(error), self.mainWin)
        dlg.show_all()
        dlg.run_and_close()
        
        if self.running:
            gtk.mainquit()
        sys.exit(1)


    def setBusyCursor(self):
        cursor = gtk.cursor_new (GDK.WATCH)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def setArrowCursor(self):
        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

    
    def onPrivacyPagePrepare(self, page, dummy):
        if not self.initPrivacy:
            self.setBusyCursor()

            privacyArea = self.xml.get_widget("privacyArea")
            privacyArea.delete_text(0, privacyArea.get_length())
            try:
                text = rhnreg.privacyText()
            except rhnreg.CommunicationError, e:
                self.mainWin.hide()
                self.fatalError(_("Fatal error retrieving privacy statement:\n") + e.errmsg)
            
            privacyArea.insert_defaults(text)
            self.initPrivacy = TRUE
            self.setArrowCursor()
            

    def onInfoPagePrepare(self, page, dummy):
        pass


    def onInfoPageNext(self, page, dummy):
        pw1 = self.xml.get_widget("passwordEntry")
        pw2 = self.xml.get_widget("password2Entry")
        uname = self.xml.get_widget("usernameEntry")
        orgid = self.xml.get_widget("orgIdEntry")
        orgpassword = self.xml.get_widget("orgPasswordEntry")
        druid = self.xml.get_widget("druid")
        
        self.setBusyCursor()
        
        # validate / check user name
        if uname.get_text() == "":
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("You must enter a user name."),
                                            self.mainWin)
            uname.grab_focus()
            return TRUE
        if pw1.get_text() == "" or pw2.get_text() == "":
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("You must enter and verify a password."),
                                            self.mainWin)
            pw1.grab_focus()
            return TRUE

        if not pw1.get_text() == pw2.get_text():
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("The two passwords you entered do not match.\nPlease re-type the second password."), self.mainWin)
            pw2.set_text("")
            pw2.grab_focus()
            return TRUE

        #FIXME, if they send a orgid, verify they send a org password as well

        try:
            self.alreadyRegistered = rhnreg.reserveUser(uname.get_text(),
                                                        pw1.get_text())
        except rhnreg.ValidationError, e:
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(wrap_text(_("The server indicated an error:\n") + e.errmsg), self.mainWin)
            uname.grab_focus()
            return TRUE
        except rhnreg.CommunicationError, e:
            self.setArrowCursor()
            self.fatalError(_("There was an error communicating with the registration server.  The message was:\n") + e.errmsg)
            
        entry = self.xml.get_widget("emailEntry")
        if not rhnreg.validateEmail(entry.get_text()):
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("You must enter a valid e-mail address."), self.mainWin)
            entry.set_text("")
            entry.grab_focus()
            return TRUE

        self.setArrowCursor()

        # skip over the personal information (productPage) if the user
        # has already registered with redhat.com.
        if self.alreadyRegistered:
           druid.set_page(self.xml.get_widget("profilePage"));
           return TRUE
            
        return FALSE


    def onProductPagePrepare(self, page, dummy):
        self.setBusyCursor()

        self.xml.get_widget("regInfoLabel").set_text(_("All information is optional."))
        self.xml.get_widget("redLabel").hide()

        label = self.xml.get_widget("redLabel")
        label.set_text("")
        style = label.get_style().copy()
        label = self.xml.get_widget("firstNameLabel")
        label.set_style(style)
        label = self.xml.get_widget("lastNameLabel")
        label.set_style(style)
        label = self.xml.get_widget("address1Label")
        label.set_style(style)
        label = self.xml.get_widget("cityLabel")
        label.set_style(style)
        label = self.xml.get_widget("stateLabel")
        label.set_style(style)
        label = self.xml.get_widget("zipLabel")
        label.set_style(style)
        label = self.xml.get_widget("countryLabel")
        label.set_style(style)
        label = self.xml.get_widget("phoneLabel")
        label.set_style(style)

        self.setArrowCursor()

        
    def onProductPageNext(self, page, dummy):
        self.productInfo = {}
        self.setBusyCursor()
        
        self.productInfo['title'] = self.xml.get_widget("titleCombo").entry.get_text()
        entry = self.xml.get_widget("firstNameEntry")

        self.productInfo['first_name'] = entry.get_text()
        entry = self.xml.get_widget("lastNameEntry")

        self.productInfo['last_name'] = entry.get_text()

        self.productInfo['company'] = self.xml.get_widget("companyEntry").get_text()

        self.productInfo['position'] = self.xml.get_widget("positionEntry").get_text()

        entry = self.xml.get_widget("address1Entry")
        self.productInfo['address1'] = entry.get_text()
        
        self.productInfo['address2'] = self.xml.get_widget("address2Entry").get_text()

        entry = self.xml.get_widget("cityEntry")
        self.productInfo['city'] = entry.get_text()

        entry = self.xml.get_widget("stateCombo").entry
        self.productInfo['state'] = entry.get_text()

        entry = self.xml.get_widget("zipEntry")
        self.productInfo['zip'] = entry.get_text()

        entry = self.xml.get_widget("countryCombo").entry
        self.productInfo['country'] = entry.get_text()

        entry = self.xml.get_widget("phoneEntry")
        self.productInfo['phone'] = entry.get_text()

        self.productInfo['fax'] = self.xml.get_widget("faxEntry").get_text()

        button = self.xml.get_widget("emailContactButton")
        if button.get_active():
            self.productInfo['contact_email'] = 1
        else:
            self.productInfo['contact_email'] = 0

        # validate mail contact
        button = self.xml.get_widget("mailContactButton")
        if (button.get_active() and
            (self.xml.get_widget("firstNameEntry").get_text() == "" or
             self.xml.get_widget("lastNameEntry").get_text() == "" or
             self.xml.get_widget("address1Entry").get_text() == "" or
             self.xml.get_widget("cityEntry").get_text() == "" or
             self.xml.get_widget("stateCombo").entry.get_text() == "" or
             self.xml.get_widget("zipEntry").get_text() == "")):
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("In order to receive communication by mail from Red Hat, you must fill out your\nfull name and address."), self.mainWin)
            return TRUE

        if button.get_active():
            self.productInfo['contact_mail'] = 1
        else:
            self.productInfo['contact_mail'] = 0

        # validate information for phone contact
        button = self.xml.get_widget("phoneContactButton")
        if (button.get_active() and
            (self.xml.get_widget("phoneEntry").get_text() == "" or
             self.xml.get_widget("firstNameEntry").get_text() == "")):
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("In order to have Red Hat contact you by phone, you must provide\nat least phone number and a name."), self.mainWin)
            if self.xml.get_widget("firstNameEntry").get_text() == "":
                self.xml.get_widget("firstNameEntry").grab_focus()
            else:
                self.xml.get_widget("phoneEntry").grab_focus()
            return TRUE

        if button.get_active():
            self.productInfo['contact_phone'] = 1
        else:
            self.productInfo['contact_phone'] = 0

        # validate information for phone contact
        button = self.xml.get_widget("faxContactButton")
        if (button.get_active() and
            (self.xml.get_widget("faxEntry").get_text() == "" or
             self.xml.get_widget("firstNameEntry").get_text() == "")):
            self.setArrowCursor()
            dlg = gnome.ui.GnomeErrorDialog(_("In order to have Red Hat contact you by fax, you must provide\nat least fax number and a name."), self.mainWin)
            if self.xml.get_widget("firstNameEntry").get_text() == "":
                self.xml.get_widget("firstNameEntry").grab_focus()
            else:
                self.xml.get_widget("faxEntry").grab_focus()
            return TRUE
        
        if button.get_active():
            self.productInfo['contact_fax'] = 1
        else:
            self.productInfo['contact_fax'] = 0
        
        button = self.xml.get_widget("utbContactButton")
        if button.get_active():
            self.productInfo['newsletter'] = 1
        else:
            self.productInfo['newsletter'] = 0

        button = self.xml.get_widget("specialContactButton")
        if button.get_active():
            self.productInfo['special_offers'] = 1
        else:
            self.productInfo['special_offers'] = 0

        # in the optional case where nothing has been entered,
        # fix up fields that had default choices:
        if self.xml.get_widget("firstNameEntry").get_text() == "":
            self.productInfo['title'] = ""
            self.productInfo['state'] = ""
            self.productInfo['country'] = ""
            
        self.setArrowCursor()


    def onProfilePagePrepare(self, page, dummy):
        self.sendHardware = self.xml.get_widget("enableHardwareButton").get_active()
        self.setBusyCursor()
        
        if not self.initProfile:
            hostname = gethostname()

            if string.find(hostname, "localhost") == -1:
                self.xml.get_widget("profileNameEntry").set_text(hostname)

            # Read all hardware in
            self.hardware = hardware.Hardware()
            
            for hw in self.hardware:
                if hw['class'] == 'CPU':
                    label = self.xml.get_widget("cpuLabel")
                    label.set_text(hw['model'])
                    label = self.xml.get_widget("speedLabel")
                    label.set_text(_("%d MHz") % hw['speed'])
                elif hw['class'] == 'MEMORY':
                    label = self.xml.get_widget("ramLabel")
                    try:
                        label.set_text(_("%s megabytes") % hw['ram'])
                    except:
                        pass

                elif hw['class'] == 'NETINFO':
                    label = self.xml.get_widget("hostnameLabel")
                    try:
                        label.set_text(hw['hostname'])
                    except:
                        pass
                    label = self.xml.get_widget("ipLabel")
                    try:
                        label.set_text(hw['ipaddr'])
                    except:
                        pass

            label = self.xml.get_widget("versionLabel")
            label.set_text(rhnreg.getVersion())

            self.initProfile = TRUE

            self.setArrowCursor()
        
    def onProfilePageNext(self, page, dummy):
        if self.xml.get_widget("profileNameEntry").get_text() == "":
            dlg = gnome.ui.GnomeErrorDialog(_("You must choose a name for this profile."), self.mainWin)
            self.xml.get_widget("profileNameEntry").grab_focus()
            return TRUE
        
        return FALSE


    def onProfilePageBack(self, page, dummy):
        if self.alreadyRegistered:
            self.xml.get_widget("druid").set_page(self.xml.get_widget("infoPage"))
            return TRUE
        
        return FALSE
    

    def onEnableHardwareButtonToggled(self, button):
        self.sendHardware = button.get_active()
        
        if not self.sendHardware:
            self.xml.get_widget("hostnameTag").hide()
            self.xml.get_widget("hostnameLabel").hide()
            self.xml.get_widget("ipTag").hide()
            self.xml.get_widget("ipLabel").hide()
            self.xml.get_widget("cpuTag").hide()
            self.xml.get_widget("cpuLabel").hide()
            self.xml.get_widget("speedTag").hide()
            self.xml.get_widget("speedLabel").hide()
            self.xml.get_widget("ramTag").hide()
            self.xml.get_widget("ramLabel").hide()
            self.xml.get_widget("hwNoteLabel").hide()
        else:
            self.xml.get_widget("hostnameTag").show()
            self.xml.get_widget("hostnameLabel").show()
            self.xml.get_widget("ipTag").show()
            self.xml.get_widget("ipLabel").show()
            self.xml.get_widget("cpuTag").show()
            self.xml.get_widget("cpuLabel").show()
            self.xml.get_widget("speedTag").show()
            self.xml.get_widget("speedLabel").show()
            self.xml.get_widget("ramTag").show()
            self.xml.get_widget("ramLabel").show()
            self.xml.get_widget("hwNoteLabel").show()
                    
    
    def onPackagePagePrepare(self, page, dummy):
        self.sendPackages = self.xml.get_widget("enableRpmButton").get_active()
        
        if not self.initPackages:
            self.setBusyCursor()
            pwin = progress.Progress()
            pwin.setLabel(_("The Registration Wizard is building a list of RPM packages installed on your system.  Please wait."))

            packages = rhnreg.buildPackageList(pwin.setProgress)
            pwin.hide()

            for package in packages:
                self.packageArea.append_row((package[0], package[1], package[2]),
                                            TRUE, package[0])

            self.initPackages = TRUE
            self.setArrowCursor()


    def onEnableRpmButtonToggled(self, button):
        self.sendPackages = button.get_active()

        if not self.sendPackages:
            self.packageArea.set_sensitive(FALSE)
            self.xml.get_widget("packageLabel").set_sensitive(FALSE)
            self.xml.get_widget("packageNoteLabel").hide()
            pass
        else:
            self.packageArea.set_sensitive(TRUE)
            self.xml.get_widget("packageLabel").set_sensitive(FALSE)
            self.xml.get_widget("packageNoteLabel").show()
            pass

    def onSendPageBack(self, page, dummy):
        return FALSE


    def onSendPageNext(self, page, dummy):
        self.setBusyCursor()

        pwin = progress.Progress()
        pwin.setLabel(_("Sending your profile information to Red Hat Network.  Please wait."))
        username = self.xml.get_widget("usernameEntry").get_text()
        password = self.xml.get_widget("passwordEntry").get_text()
        email = self.xml.get_widget("emailEntry").get_text()
        orgId = self.xml.get_widget("orgIdEntry").get_text()
        orgIdPassword = self.xml.get_widget("orgPasswordEntry").get_text()
        profileName = self.xml.get_widget("profileNameEntry").get_text()

        if orgId == "":
            orgId = None
        if orgIdPassword == "":
            orgIdPassword = None

        try:
            rhnreg.registerUser(username, password, email, orgId, orgIdPassword)
        except rhnreg.CommunicationError, e:
            pwin.hide()
            self.fatalError(_("Problem registering user name:\n") + e.errmsg)
        except:
            self.setArrowCursor()
            pwin.hide()
            dlg = gnome.ui.GnomeErrorDialog(_("Problem registering username."),
                                            self.mainWin)
            return TRUE

        pwin.setProgress(1, 5)

        try:
            systemId = rhnreg.registerSystem(username, password, profileName)
        except rhnreg.CommunicationError, e:
            pwin.hide()
            self.fatalError(_("Problem registering system:\n") + e.errmsg)
        except:
            self.setArrowCursor()
            pwin.hide()
            dlg = gnome.ui.GnomeErrorDialog(_("Problem registering system."),
                                            self.mainWin)
            return TRUE

        pwin.setProgress(2, 5)

        # write the system id out.
        if not rhnreg.writeSystemId(systemId):
            self.setArrowCursor()
            pwin.hide()
            dlg = gnome.ui.GnomeErrorDialog(_("Problem writing out system id to disk."),
                                            self.mainWin)
            return TRUE

        # incorporate the info from the oemInfoFile as well
        self.oemInfo = rhnreg.getOemInfo()

        # dont send if already registered, do send if they have oemInfo
        if ( not self.alreadyRegistered ) or ( len(self.oemInfo) ):
            # send product registration information
            try:
                rhnreg.registerProduct(systemId, self.productInfo, self.oemInfo)
            except rhnreg.CommunicationError, e:
                pwin.hide()
                self.fatalError(_("Problem registering personal information:\n") + e.errmsg)
            except:
                self.setArrowCursor()
                pwin.hide()
                dlg = gnome.ui.GnomeErrorDialog(_("Problem registering personal information"), self.mainWin)
                return TRUE

                
        pwin.setProgress(3, 5)

        # maybe upload hardware profile
        if self.sendHardware:
            try:
                rhnreg.sendHardware(systemId, self.hardware)
            except rhnreg.CommunicationError, e:
                pwin.hide()
                self.fatalError(_("Problem sending hardware profile:\n") + e.errmsg)
            except:
                self.setArrowCursor()
                pwin.hide()
                dlg = gnome.ui.GnomeErrorDialog(_("Problem sending hardware profile."),
                                                self.mainWin)
                return TRUE

        pwin.setProgress(4, 5)
        
        # build up package list if necessary
        if self.sendPackages:

            packageList = rhnreg.buildPackageList(lambda amount, total: gtk.mainiteration(FALSE))
            selection = []
            for row in range(self.packageArea.n_rows):
                rowData = self.packageArea.get_row_data(row)
                if rowData[0] == 1:
                    selection.append(rowData[1])

            selectedPackages = []
            for pkg in packageList:
                if pkg[0] in selection:
                    selectedPackages.append(pkg)

            try:
                rhnreg.sendPackages(systemId, selectedPackages)
            except rhnreg.CommunicationError, e:
                pwin.hide()
                self.fatalError(_("Problem sending packages:\n") + e.errmsg)
            except:
                self.setArrowCursor()
                pwin.hide()
                dlg = gnome.ui.GnomeErrorDialog(_("Problem sending packages."),
                                                self.mainWin)
                return TRUE

        pwin.setProgress(5, 5)
        pwin.hide()
        
        self.setArrowCursor()
        return FALSE


    def onFinishPagePrepare(self, page, dummy):
        druid = self.xml.get_widget("druid")
        druid.set_buttons_sensitive(FALSE, TRUE, FALSE)
        return FALSE


    def onFinishPageFinish(self, page, dummy):
        gtk.mainquit()


def main():

    def cb(foo):
        return foo
        
    test = 0
    if len(saveargs) > 0:
        if saveargs[0] == "-t" or saveargs[0] == "--test":
            test = 1
        if saveargs[0] == "-a" or saveargs[0] == "--about":
            dlg = gnome.ui.GnomeAbout(_("About Red Hat Network Registration"),
                                      "1.2",
                                      "Copyright (c) 2000-2001 Red Hat, Inc.",
                                      ["Preston Brown <pbrown@redhat.com>",
                                       "Cristian Gafton <gafton@redhat.com>",
                                       "Adrian Likins <alikins@redhat.com>"],
                                      _("Please Report bugs to Red Hat's Bug Tracking System: http://bugzilla.redhat.com/bugzilla/"))
            dlg.run_and_close()
            sys.exit(0)
        
    signal.signal (signal.SIGINT, signal.SIG_DFL)

    if not rhnreg.hasSSL():
        dlg = gnome.ui.GnomeQuestionDialog(_("""Your system libraries do not support SSL (secure) connections.  Any data
that you send or receive from Red Hat Network will be transmitted in the clear.
Continue anyway?"""), cb)
        if dlg.run_and_close():
            sys.exit(1)

    if geteuid() != 0 and not test:
        dlg = gnome.ui.GnomeErrorDialog(_("You must run the Red Hat Network registration program as root."))
        dlg.run_and_close()
        sys.exit(1)

    if rhnreg.registered() and not test:
        dlg = gnome.ui.GnomeQuestionDialog(_("This system appears to already be registered.\nContinue anyway?"), cb)
        if dlg.run_and_close():
            sys.exit(1)

    try:
        from gtk import _disable_gdk_threading
        _disable_gdk_threading()
    except ImportError:
        pass


    gui = Gui()
        
    if test:
        gui.test = TRUE
    
    gtk.mainloop()

if __name__ == "__main__":
    main()
